#coding: utf-8
u'''
Загрузчики сохраненных запросов.
'''

import json
from request import MockRequest


class RequestListLoader(object):
    u'''
    Интерфейс класса, ответственного за загрузку сохраненных запросов из файла.

    .. method:: RequestListLoader.load_requests(self, fp)

    Загрузчик информации о сохраненных запросах из файла.

    :param fp: Файловый дескриптор
    '''

    def load_requests(self, fp):
        raise NotImplemented


class DebuggieLoader(RequestListLoader):
    u'''
    Загрузчик сохранных запросов из файла, созданного утилитой :mod:`debuggie`.

    .. method:: DebuggieLoader.load_requests(self, fp)

    Загружает информацию о перехваченных :mod:`debuggie` запросах
    из файла, который представляет из себя набор строчек с сериализованным
    словарем в каждой из них. В этом же файле сохранены данные о полученных
    ответах и вызванных исключениях, но мы игнорируем эти записи.

    Функция является итератором, считывая и десериализуя строчки только
    по мере надобности.

    :param fp: Файловый дескриптор
    '''

    def load_requests(self, fp):
        while True:
            # Построчно считываем каждый запрос из файла
            line = fp.readline()
            if not line:
                break
            rec = json.loads(line)

            # Нас интересуют только записанные запросы, поэтому отфильтровываем
            # записанные ответы и пойманные исключения.
            if 'action' in rec and rec['action']['type'] == 'request':
                username, uid = rec['user']

                req = MockRequest(
                    rec['action']['data']['verb'],
                    rec['url'],
                    body=rec['action']['data'].get('body', ''),
                    cookies=rec['action']['data']['cookies'],
                    headers=rec['action']['data'].get('headers', {}))
                req.auth['username'] = username

                yield req
